/*
 * Copyright 2018 The Trickster Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package clickhouse

import (
	"net/http"

	bo "github.com/tricksterproxy/trickster/pkg/backends/options"
	"github.com/tricksterproxy/trickster/pkg/proxy/paths/matching"
	po "github.com/tricksterproxy/trickster/pkg/proxy/paths/options"
)

func (c *Client) RegisterHandlers(map[string]http.Handler) {

	c.TimeseriesBackend.RegisterHandlers(
		map[string]http.Handler{
			// This is the registry of handlers that Trickster supports for ClickHouse,
			// and are able to be referenced by name (map key) in Config Files
			"health": http.HandlerFunc(c.HealthHandler),
			"query":  http.HandlerFunc(c.QueryHandler),
			"proxy":  http.HandlerFunc(c.ProxyHandler),
		},
	)
}

// DefaultPathConfigs returns the default PathConfigs for the given Provider
func (c *Client) DefaultPathConfigs(o *bo.Options) map[string]*po.Options {
	paths := map[string]*po.Options{
		"/": {
			Path:           "/",
			HandlerName:    "query",
			Methods:        []string{http.MethodGet, http.MethodPost},
			MatchType:      matching.PathMatchTypePrefix,
			MatchTypeName:  "prefix",
			CacheKeyParams: []string{"query", "database"},
		},
	}
	return paths
}
